==> maxdb-all-linux-64bit-x86_64-7_6_00_36.tgz: SAPDB/Install/App/UnregisterPackages.pm <==
#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_36/sys/src/install/perl/SAPDB/Install/App/UnregisterPackages.pm#1 $
# $DateTime: 2006/11/17 17:24:42 $
# $Change: 140017 $
#
# Desc: application to call register method of each installed MaxDB package
#		this is useful if registrations has been deleted


package DummyLog;

sub new{
	my $self = bless({},shift);
	my $file = shift;
	if($file =~ /\S/){
		if ($^O =~ /mswin/i){
			unless ($file =~ /^[a-z]:|^\\|^\//i){
				my $callerdir = $SAPDB::Install::Config{'CallerDir'};
				$callerdir =~ s/\\/\//g;
				$file = $callerdir.'/'.$file; 
			}
		}
		else{
			unless ($file =~ /^\//){
				$file = $SAPDB::Install::Config{'CallerDir'}.'/'.$file
			}
		}	
		open(LD,'>'.$file) or return undef;
		$self->{'file'} = $file;
	}
	return $self;
}

sub SetMsg{
	my ($self,$text) = @_;
	if(exists $self->{'file'}){
		print LD $text;
	}
	return 1;
}

sub DESTROY{
	my ($self) = @_;
	if(exists $self->{'file'}){
		close(LD);
		delete $self->{'file'};
	}	
}

1;


package SAPDB::Install::App::UnregisterPackages;

use SDB::Common::Getopt::Long;
use SAPDB::Install::Registry;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Values;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;


sub mydiesoft{
	die($SAPDB::Install::StdIO::err);
}


sub cleanGlobals{
	
	#
	# killing global references for clean destruction
	#
	
	@SAPDB::Install::InstRegistry::Package::SELFOBJ = ();
	$SAPDB::Install::Values::log = undef;		
}

sub main{
	local @ARGV = @_;
	my $rc = 0;
	my ($opt_logfile,$opt_database,$opt_sw_all,
		$opt_sw_kernel,$opt_sw_all_kernel,$opt_substitute_drive);
	my $opt_database;
	
	unless(IsAdmin()){
		print STDERR "please restart as ".($^O =~ /mswin/i ? 'local administrator' : 'root')."!\n";
		return -1;
	}
	my %opt_ctrl = (
		'logfile=s' => \$opt_logfile,
		'd=s' => \$opt_database,
		'sw_all' => \$opt_sw_all,
		'sw_kernel' => \$opt_sw_kernel,
		'R=s' => \$opt_dbroot,				
		'sw_all_kernel' => \$opt_sw_all_kernel
	);	
	
	if ($^O =~ /mswin/i){
		$opt_ctrl{'-substitute_drive'} = \$opt_substitute_drive;
	}	
		
	sub OptCheck{
		my $rc = 1;
		if (!defined $opt_sw_all && 
			!defined $opt_sw_all_kernel &&
			!defined $opt_sw_kernel && 
			!defined $opt_database){
			print "specify at least one of the following options: -sw_all, -sw_all_kernel, -sw_kernel, -d <database>\n";
			$rc = 0;
		}
			
		return $rc;
	}
		
		
	if (!GetOptions(%opt_ctrl) || !OptCheck() ){
		return 42;
	}	
		
	my ($data,$prog) = readIndepPath();
	SDB::Common::Getopt::Long::Configure ('no_pass_through','no_ignore_case');
		
	my $log = DummyLog->new($opt_logfile);
	unless (defined $log){
		print2stderr("cannot create logfile \"$opt_logfile\": $!\n");
		return -1;
	}
		
	
	$SAPDB::Install::Values::log = $log;
	
	if ($data !~ /\S/){
		print2stderr ("no installation found\n");
		cleanGlobals();
		return -1;
	}
	
	unless(-f $data.'/config/install/INSTREG'){
		print2stderr ("InstallRegistry not found\n");
		cleanGlobals();
		return -1;
	}
			
	my $instreg = SAPDB::Install::InstallRegistry->new($data.'/config/install',0,$log);
	
	unless (defined $instreg){
		print2stderr ("cannot open InstallRegistry\n");
		return -1;
	}
	
	if ($opt_substitute_drive){
		my %releases = $instreg->getInstallPathes('Base');
		my ($orig_indep_programs) = keys(%releases); 
		$log->SetMsg("MSG: original independent program path is \"$orig_indep_programs\"\n");
		my %regdata = $instreg->getPackageData('Base',normalizePath($orig_indep_programs));
		my $orig_indep_data = $regdata{'Pathes'}->[0]->{'value'};
		$log->SetMsg("MSG: original independent data path is \"$orig_indep_data\"\n");
		normalizePath(\$orig_indep_data);
		my $pattern = $orig_indep_data;
		$pattern =~ s/^[a-z]://; 
		$pattern =~ s/\//\\\//g;
		$pattern = '^(.+)'.$pattern;
		my $new_indep_data = normalizePath($data);
		my ($prefix) = ($new_indep_data =~ /$pattern/);
		$log->SetMsg("MSG: new installation prefix is \"$prefix\"\n");
		$instreg->SubstituteDrive($prefix);
	}	
	
	my %regdata = $instreg->getPackageData('Base',normalizePath($prog));
	
	my $indep_data = $regdata{'Pathes'}->[0]->{'value'};
	
	unless(normalizePath($indep_data) eq normalizePath($data)){
		print2stderr ("Independent data path is not unique: ".
					"\"$indep_data\" vs. \"$data\"\n");
		cleanGlobals();
		return -1;
	}
	
		
	
	#
	# overload diesoft in SAPDB::Install::System	
	# to avoid call of exit()
	#	
	#	
	
	if(exists ${'SAPDB::Install::System::'}{'diesoft'}){
		*SAPDB::Install::System::diesoft = 
			\&mydiesoft;
	}	

	my $kernel_root;
	
	if ($opt_database){
		require SAPDB::Install::Service::DBM;
		my $dbm = SAPDB::Install::Service::DBM->new($opt_database);
		unless ($dbm->{'registered'}){
			print2stderr ($dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		if (defined $dbm){
			if ($opt_sw_kernel){
				$kernel_root = $dbm->get_dbroot ();
				unless (defined $kernel_root){
					print2stderr ($dbm->{'last_error'}."\n");
					cleanGlobals();
					return -1;
				}
			}
		}
		else{
			print2stderr ("cannot connect to database $opt_database\n");
			cleanGlobals();
			return -1;
		}
		unless (defined $dbm->db_state){
			print2stderr ("cannot unregister database $opt_database: ".$dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		
		if ($dbm->{'state'} ne 'OFFLINE'){
			print2stdout ("database ".$opt_database." is running\n");
			if(defined $dbm->db_offline()){
				print2stdout ("database shutdown finished successfully\n");
			}
			else{
				print2stderr ("cannot stop database: ".$dbm->{'last_error'}."\n");
				cleanGlobals();
				return -1;
			}
		}
		
		if(defined $dbm->db_unreg()){
			print2stdout ("database $opt_database unregistered successfully\n");
		}
		else{
			print2stderr ("cannot unregister database: ".$dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		
	}
	else{
		if (defined $opt_dbroot){
			$kernel_root = $opt_dbroot;
		}
	}
	
	
	my $collector = SAPDB::Install::PackageCollector->new();
	

	$collector->log($log);
	
		
	unless(defined $collector->GenRegPackageTree($instreg)){
		print2stderr ("cannot create package tree: ".$collector->last_err."\n");
		cleanGlobals();
		return -1;
	}

	sub select_tree{
		my ($node) = @_;
		$node->{'selected'} = 1; 
		return 1 unless defined $node->{'children'};
		foreach my $child (@{$node->{'children'}}){
			select_tree($child);
		}
	}
	
	sub find_tree{
		my($node,$packagename,$path) = @_;
		if ($node->{'name'} eq $packagename){
			if (defined $path){
				if (normalizePath($path) eq $node->{'instkey'}){
					return $node;
				}
			}
			else{
				return $node;	
			}
		
		}
		else{
			foreach my $child (@{$node->{'children'}}){
				my $found = find_tree($child,$packagename,$path);
				return $found if (defined $found);
			}
		}
		return undef;	
	}
	
	my $root_node;
	
	if ($opt_sw_all){
	
		#
		#	select all packages	
		#
		
		$root_node = $collector->regpackage_tree;
	}
	elsif($opt_sw_all_kernel){
		
		#
		# select all kernel packages and Server Utilities
		#
		
		$root_node = find_tree($collector->regpackage_tree,'Server Utilities');
		
		
	}
	elsif($opt_sw_kernel){
		
		#
		# select kernel package of database <db name>
		#
	
		$root_node = find_tree($collector->regpackage_tree,'Database Kernel',$kernel_root);
		
		unless (defined $root_node){
			print2stderr("no database kernel found in dependent path \"$kernel_root\"\n");
			cleanGlobals();
			return -1;
		}
	}
	
	select_tree ($root_node) if defined $root_node;


	#
	#	getting packages
	#		
	
	my $packages = $collector->GenRegPackages($instreg);
	unless(defined $packages){
		print2stderr ("cannot get packages: ".$collector->last_error."\n");
		cleanGlobals();
		return -1;		
	}
	
	#
	#	check existing databases 
	#
	
	my @kernels_with_db = ();
	my $server_utilities_selected = 0;
	
	foreach my $package (@$packages){
		if ($package->Name eq 'Server Utilities'){
			$server_utilities_selected = 1;
		}
		if ($package->Name eq 'Database Kernel'){
			my %dbroots = getInstances();
			foreach my $DB	(keys(%dbroots)){
				if(normalizePath($dbroots{$DB}) eq $package->Path){
					print2stderr("found instance $DB \n");
					push @kernels_with_db,$package->Path;
				}
			}
		}
	}	
		
	if($#kernels_with_db > -1){
		print2stderr("cannot unregister Database Kernel (\'".
					join('\',\'',@kernels_with_db).
					"\'): first you have to unregister or drop all instances\n");
		return -1;
	}
	
	
	if ($^O !~ /mswin/i and $server_utilities_selected){
		require SAPDB::Install::Service::Communication;
		my $x_server = SAPDB::Install::Service::Communication->new();
		if ($x_server->state()->{'is_running'}){
			print "x_server is running\n";
			my $rc = $x_server->stop();
			unless (defined $rc){
				print2stderr("Cannot stop MaxDB communication service (x_server):".
							  $x_server->last_error()."\n");
				cleanGlobals();
				return -1;
			}
			print2stdout ("x_server stopped\n");
		}
		else{
			print2stdout ("x_server isn\'t running\n");
		}
	}
	
	
	foreach my $package (@$packages){
		my $name = $package->DispName;
		$package->evalScript;		
		
		#
		# overload diesoft in package script		
		# to avoid call of exit()	
		#	

		if(exists ${'SAPDB::Install::Uninst'.$package->ID.'::'}{'diesoft'}){
			*{'SAPDB::Install::Uninst'.$package->ID.'::diesoft'} =
				\&mydiesoft;
		}
		
		
		#
		# call unregister()
		#

		if(defined $package->unregister){
			print "unregistering package $name [".$package->Path."]\n";
			eval{
				&{$package->unregister}();
			};
			if($@){
				$rc = -1;
				print2stderr("unregistering $name [".$package->Path."] failed\n");
			};	
		}
	}
	if ($opt_sw_all){
		deleteSAPDBTech({'keep_user_group' => 1});
		if ($^O =~ /mswin/i){
			removeFromSysPath("$prog/bin;$prog/pgm");
			removeUninstallEntry();
		}
		
	}		
	cleanGlobals();
	return $rc;
}

==> maxdb-all-linux-64bit-x86_64-7_6_03_15.tgz: SAPDB/Install/App/UnregisterPackages.pm <==
#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_03/b_15/sys/src/install/perl/SAPDB/Install/App/UnregisterPackages.pm#1 $
# $DateTime: 2008/01/18 16:05:39 $
# $Change: 173096 $
#
# Desc: application to call register method of each installed MaxDB package
#		this is useful if registrations has been deleted


package DummyLog;

sub new{
	my $self = bless({},shift);
	my $file = shift;
	if($file =~ /\S/){
		if ($^O =~ /mswin/i){
			unless ($file =~ /^[a-z]:|^\\|^\//i){
				my $callerdir = $SAPDB::Install::Config{'CallerDir'};
				$callerdir =~ s/\\/\//g;
				$file = $callerdir.'/'.$file; 
			}
		}
		else{
			unless ($file =~ /^\//){
				$file = $SAPDB::Install::Config{'CallerDir'}.'/'.$file
			}
		}	
		open(LD,'>'.$file) or return undef;
		$self->{'file'} = $file;
	}
	return $self;
}

sub SetMsg{
	my ($self,$text) = @_;
	if(exists $self->{'file'}){
		print LD $text;
	}
	return 1;
}

sub DESTROY{
	my ($self) = @_;
	if(exists $self->{'file'}){
		close(LD);
		delete $self->{'file'};
	}	
}

1;


package SAPDB::Install::App::UnregisterPackages;

use SDB::Common::Getopt::Long;
use SAPDB::Install::Registry;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Values;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;


sub mydiesoft{
	die($SAPDB::Install::StdIO::err);
}


sub cleanGlobals{
	
	#
	# killing global references for clean destruction
	#
	
	@SAPDB::Install::InstRegistry::Package::SELFOBJ = ();
	$SAPDB::Install::Values::log = undef;		
}

sub main{
	local @ARGV = @_;
	my $rc = 0;
	my ($opt_logfile,$opt_database,$opt_sw_all,
		$opt_sw_kernel,$opt_sw_all_kernel,$opt_substitute_drive);
	my $opt_database;
	
	unless(IsAdmin()){
		print STDERR "please restart as ".($^O =~ /mswin/i ? 'local administrator' : ( isPASE() ? 'QSECOFR' : 'root') )."!\n";
		return -1;
	}
	my %opt_ctrl = (
		'logfile=s' => \$opt_logfile,
		'd=s' => \$opt_database,
		'sw_all' => \$opt_sw_all,
		'sw_kernel' => \$opt_sw_kernel,
		'R=s' => \$opt_dbroot,				
		'sw_all_kernel' => \$opt_sw_all_kernel
	);	
	
	if ($^O =~ /mswin/i){
		$opt_ctrl{'-substitute_drive'} = \$opt_substitute_drive;
	}	
		
	sub OptCheck{
		my $rc = 1;
		if (!defined $opt_sw_all && 
			!defined $opt_sw_all_kernel &&
			!defined $opt_sw_kernel && 
			!defined $opt_database){
			print "specify at least one of the following options: -sw_all, -sw_all_kernel, -sw_kernel, -d <database>\n";
			$rc = 0;
		}
			
		return $rc;
	}
		
		
	if (!GetOptions(%opt_ctrl) || !OptCheck() ){
		return 42;
	}	
		
	my ($data,$prog) = readIndepPath();
	SDB::Common::Getopt::Long::Configure ('no_pass_through','no_ignore_case');
		
	my $log = DummyLog->new($opt_logfile);
	unless (defined $log){
		print2stderr("cannot create logfile \"$opt_logfile\": $!\n");
		return -1;
	}
		
	
	$SAPDB::Install::Values::log = $log;
	
	if ($data !~ /\S/){
		print2stderr ("no installation found\n");
		cleanGlobals();
		return -1;
	}
	
	unless(-f $data.'/config/install/INSTREG'){
		print2stderr ("InstallRegistry not found\n");
		cleanGlobals();
		return -1;
	}
			
	my $instreg = SAPDB::Install::InstallRegistry->new($data.'/config/install',0,$log);
	
	unless (defined $instreg){
		print2stderr ("cannot open InstallRegistry\n");
		return -1;
	}
	
	if ($opt_substitute_drive){
		my %releases = $instreg->getInstallPathes('Base');
		my ($orig_indep_programs) = keys(%releases); 
		$log->SetMsg("MSG: original independent program path is \"$orig_indep_programs\"\n");
		my %regdata = $instreg->getPackageData('Base',normalizePath($orig_indep_programs));
		my $orig_indep_data = $regdata{'Pathes'}->[0]->{'value'};
		$log->SetMsg("MSG: original independent data path is \"$orig_indep_data\"\n");
		normalizePath(\$orig_indep_data);
		my $pattern = $orig_indep_data;
		$pattern =~ s/^[a-z]://; 
		$pattern =~ s/\//\\\//g;
		$pattern = '^(.+)'.$pattern;
		my $new_indep_data = normalizePath($data);
		my ($prefix) = ($new_indep_data =~ /$pattern/);
		$log->SetMsg("MSG: new installation prefix is \"$prefix\"\n");
		$instreg->SubstituteDrive($prefix);
	}	
	
	my %regdata = $instreg->getPackageData('Base',normalizePath($prog));
	
	my $indep_data = $regdata{'Pathes'}->[0]->{'value'};
	
	unless(normalizePath($indep_data) eq normalizePath($data)){
		print2stderr ("Independent data path is not unique: ".
					"\"$indep_data\" vs. \"$data\"\n");
		cleanGlobals();
		return -1;
	}
	
		
	
	#
	# overload diesoft in SAPDB::Install::System	
	# to avoid call of exit()
	#	
	#	
	
	if(exists ${'SAPDB::Install::System::'}{'diesoft'}){
		*SAPDB::Install::System::diesoft = 
			\&mydiesoft;
	}	

	my $kernel_root;
	
	if ($opt_database){
		require SAPDB::Install::Service::DBM;
		my $dbm = SAPDB::Install::Service::DBM->new($opt_database);
		unless ($dbm->{'registered'}){
			print2stderr ($dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		if (defined $dbm){
			if ($opt_sw_kernel){
				$kernel_root = $dbm->get_dbroot ();
				unless (defined $kernel_root){
					print2stderr ($dbm->{'last_error'}."\n");
					cleanGlobals();
					return -1;
				}
			}
		}
		else{
			print2stderr ("cannot connect to database $opt_database\n");
			cleanGlobals();
			return -1;
		}
		unless (defined $dbm->db_state){
			print2stderr ("cannot unregister database $opt_database: ".$dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		
		if ($dbm->{'state'} ne 'OFFLINE'){
			print2stdout ("database ".$opt_database." is running\n");
			if(defined $dbm->db_offline()){
				print2stdout ("database shutdown finished successfully\n");
			}
			else{
				print2stderr ("cannot stop database: ".$dbm->{'last_error'}."\n");
				cleanGlobals();
				return -1;
			}
		}
		
		if(defined $dbm->db_unreg()){
			print2stdout ("database $opt_database unregistered successfully\n");
		}
		else{
			print2stderr ("cannot unregister database: ".$dbm->{'last_error'}."\n");
			cleanGlobals();
			return -1;
		}
		
	}
	else{
		if (defined $opt_dbroot){
			$kernel_root = $opt_dbroot;
		}
	}
	
	
	my $collector = SAPDB::Install::PackageCollector->new();
	

	$collector->log($log);
	
		
	unless(defined $collector->GenRegPackageTree($instreg)){
		print2stderr ("cannot create package tree: ".$collector->last_err."\n");
		cleanGlobals();
		return -1;
	}

	sub select_tree{
		my ($node) = @_;
		$node->{'selected'} = 1; 
		return 1 unless defined $node->{'children'};
		foreach my $child (@{$node->{'children'}}){
			select_tree($child);
		}
	}
	
	sub find_tree{
		my($node,$packagename,$path) = @_;
		if ($node->{'name'} eq $packagename){
			if (defined $path){
				if (normalizePath($path) eq $node->{'instkey'}){
					return $node;
				}
			}
			else{
				return $node;	
			}
		
		}
		else{
			foreach my $child (@{$node->{'children'}}){
				my $found = find_tree($child,$packagename,$path);
				return $found if (defined $found);
			}
		}
		return undef;	
	}
	
	my $root_node;
	
	if ($opt_sw_all){
	
		#
		#	select all packages	
		#
		
		$root_node = $collector->regpackage_tree;
	}
	elsif($opt_sw_all_kernel){
		
		#
		# select all kernel packages and Server Utilities
		#
		
		$root_node = find_tree($collector->regpackage_tree,'Server Utilities');
		
		
	}
	elsif($opt_sw_kernel){
		
		#
		# select kernel package of database <db name>
		#
	
		$root_node = find_tree($collector->regpackage_tree,'Database Kernel',$kernel_root);
		
		unless (defined $root_node){
			print2stderr("no database kernel found in dependent path \"$kernel_root\"\n");
			cleanGlobals();
			return -1;
		}
	}
	
	select_tree ($root_node) if defined $root_node;


	#
	#	getting packages
	#		
	
	my $packages = $collector->GenRegPackages($instreg);
	unless(defined $packages){
		print2stderr ("cannot get packages: ".$collector->last_error."\n");
		cleanGlobals();
		return -1;		
	}
	
	#
	#	check existing databases 
	#
	
	my @kernels_with_db = ();
	my $server_utilities_selected = 0;
	
	foreach my $package (@$packages){
		if ($package->Name eq 'Server Utilities'){
			$server_utilities_selected = 1;
		}
		if ($package->Name eq 'Database Kernel'){
			my %dbroots = getInstances();
			foreach my $DB	(keys(%dbroots)){
				if(normalizePath($dbroots{$DB}) eq $package->Path){
					print2stderr("found instance $DB \n");
					push @kernels_with_db,$package->Path;
				}
			}
		}
	}	
		
	if($#kernels_with_db > -1){
		print2stderr("cannot unregister Database Kernel (\'".
					join('\',\'',@kernels_with_db).
					"\'): first you have to unregister or drop all instances\n");
		return -1;
	}
	
	
	if ($^O !~ /mswin/i and $server_utilities_selected){
		require SAPDB::Install::Service::Communication;
		my $x_server = SAPDB::Install::Service::Communication->new();
		if ($x_server->state()->{'is_running'}){
			print "x_server is running\n";
			my $rc = $x_server->stop();
			unless (defined $rc){
				print2stderr("Cannot stop MaxDB communication service (x_server):".
							  $x_server->last_error()."\n");
				cleanGlobals();
				return -1;
			}
			print2stdout ("x_server stopped\n");
		}
		else{
			print2stdout ("x_server isn\'t running\n");
		}
	}
	
	
	foreach my $package (@$packages){
		my $name = $package->DispName;
		$package->evalScript;		
		
		#
		# overload diesoft in package script		
		# to avoid call of exit()	
		#	

		if(exists ${'SAPDB::Install::Uninst'.$package->ID.'::'}{'diesoft'}){
			*{'SAPDB::Install::Uninst'.$package->ID.'::diesoft'} =
				\&mydiesoft;
		}
		
		
		#
		# call unregister()
		#

		if(defined $package->unregister){
			print "unregistering package $name [".$package->Path."]\n";
			eval{
				&{$package->unregister}();
			};
			if($@){
				$rc = -1;
				print2stderr("unregistering $name [".$package->Path."] failed\n");
			};	
		}
	}
	if ($opt_sw_all){
		deleteSAPDBTech({'keep_user_group' => 1});
		if ($^O =~ /mswin/i){
			removeFromSysPath("$prog/bin;$prog/pgm");
			removeUninstallEntry();
		}
		
	}		
	cleanGlobals();
	return $rc;
}
